//  Movement vector
var dx = 0,
    dy = 0;

//  Move with keyboard
if (keyboard_check_pressed(vk_up)) dy -= 1;
if (keyboard_check_pressed(vk_down)) dy += 1;
if (keyboard_check_pressed(vk_left)) dx -= 1;
if (keyboard_check_pressed(vk_right)) dx += 1;

//  Backtrack with keyboard
if (keyboard_check_pressed(vk_backspace)) {
    dx = 0;
    dy = 0;
    if (grid[# posX-1, posY] == step-1) { dx = -1; }
    if (grid[# posX+1, posY] == step-1) { dx = +1; }
    if (grid[# posX, posY-1] == step-1) { dy = -1; }
    if (grid[# posX, posY+1] == step-1) { dy = +1; }
    keyboard_key_release(vk_backspace);
}

//  No diagonals!
if (dx != 0) dy = 0;

//  Move?
if (dx != 0 || dy != 0) {
    //  Is space tile-free?
    if (tile_get_empty(tilemap_get(tilemapID, posX+dx, posY+dy))) {
        var cell = grid[# posX+dx, posY+dy];
        //  Has the cell been visited?
        if (cell == gridFree) {
            //  Move into free space
            step++;
            posX += dx;
            posY += dy;
            x = tileW * posX;
            y = tileH * posY;
            grid[# posX, posY] = step;
            update = true;
        } else if (cell == step-1) {
            //  Backtrack into previous cell
            grid[# posX, posY] = gridFree;
            posX += dx;
            posY += dy;
            x = tileW * posX;
            y = tileH * posY;
            step--;
            update = true;
        }
    }
}

//  Update arrow-path tiles as needed
if (update) {
    
    tilemap_clear(pathmapID, 0);

    var pathX = ds_grid_value_x(grid, 0, 0, gridW, gridH, 1);
    var pathY = ds_grid_value_y(grid, 0, 0, gridW, gridH, 1);
    var prevX = pathX;
    var prevY = pathY;
    var nextX = pathX;
    var nextY = pathY;
    
    for (var pathStep=1; pathStep <= step; pathStep++) {
    
        var frame = 0;
        
        //  Find the next tile among neighbors
        if (pathStep < step) {
            nextX = ds_grid_value_x(grid, pathX-1, pathY-1, pathX+1, pathY+1, pathStep+1);
            nextY = ds_grid_value_y(grid, pathX-1, pathY-1, pathX+1, pathY+1, pathStep+1);
        }
        
        //  Select a tile using binary proximity mask.
        //  If the cell has a connected neighbor to the
        //  west/east/north/south, set bits 0/1/2/3.
        frame |= ((prevX < pathX) | (nextX < pathX)) << 0;
        frame |= ((prevX > pathX) | (nextX > pathX)) << 1;
        frame |= ((prevY < pathY) | (nextY < pathY)) << 2;
        frame |= ((prevY > pathY) | (nextY > pathY)) << 3;
        
        if (pathStep == step) {
            //  Draw arrow head
            tilemap_set(pathmapID, frame+1, pathX, pathY);
        } else {
            //  Draw path
            tilemap_set(pathmapID, frame+17, pathX, pathY);
        }
        
        //  Move ahead
        prevX = pathX;
        prevY = pathY;
        pathX = nextX;
        pathY = nextY;
    }
    update = false;
}
